"use strict";

var ObjectWithListeners = require('../../ObjectWithListeners');

var Annotation = require("./Annotation");
var Model = require('./MapModel');

var logger = require('../../logger');

function Project(data) {
  // call super constructor
  ObjectWithListeners.call(this);
  this.registerListenerType("onreload");

  if (data !== undefined) {
    this.loadFromData(data);
  }
}

// this class inherits from ObjectWithListeners class where generic methods for
// listeners are set
Project.prototype = Object.create(ObjectWithListeners.prototype);
Project.prototype.constructor = Project;

Project.prototype.loadFromData = function(data) {
  if (typeof data === "string") {
    // replace is due to some strange problem with serialization
    data = JSON.parse(data.replace(/\n/g, " "));
  }
  if (data instanceof Project) {
    this.setId(data.getId());
    this.setProjectId(data.getProjectId());
    this.setVersion(data.getVersion());
    this.setName(data.getName());
    this.setOverviewImages(data.getOverviewImages());
    this.setTopOverviewImage(data.getTopOverviewImage());
    this.setDisease(data.getDisease());
    this.setOrganism(data.getOrganism());
    this.setStatus(data.getStatus());
    this.setStatus(data.getProgress());

    this.setModel(new Model(data.getModel()));

    this.callListeners("onreload");

  } else {
    this.setId(parseInt(data.idObject));
    this.setProjectId(data.projectId);
    this.setVersion(data.version);
    this.setName(data.name);
    this.setOverviewImages(data.overviewImageViews);
    this.setTopOverviewImage(data.topOverviewImage);
    this.setDisease(data.disease);
    this.setOrganism(data.organism);
    this.setStatus(data.status);
    this.setProgress(data.progress);

    this.callListeners("onreload");
  }
};

Project.prototype.getId = function() {
  return this._id;
};

Project.prototype.setId = function(id) {
  this._id = parseInt(id);
};

Project.prototype.getProjectId = function() {
  return this._projectId;
};

Project.prototype.setProjectId = function(projectId) {
  this._projectId = projectId;
};

Project.prototype.getVersion = function() {
  return this._version;
};

Project.prototype.setVersion = function(version) {
  this._version = version;
};

Project.prototype.getName = function() {
  return this._name;
};
Project.prototype.setName = function(name) {
  this._name = name;
};
Project.prototype.getModel = function() {
  return this._model;
};
Project.prototype.setModel = function(model) {
  this._model = model;
};

Project.prototype.getOverviewImages = function() {
  return this._overviewImages;
};
Project.prototype.setOverviewImages = function(overviewImages) {
  this._overviewImages = overviewImages;
};

Project.prototype.getTopOverviewImage = function() {
  return this._topOverviewImage;
};
Project.prototype.setTopOverviewImage = function(topOverviewImage) {
  this._topOverviewImage = topOverviewImage;
};

Project.prototype.getDisease = function() {
  return this._disease;
};
Project.prototype.setDisease = function(disease) {
  if (disease !== undefined && disease !== null) {
    this._disease = new Annotation(disease);
  } else {
    this._disease = undefined;
  }
};

Project.prototype.getOrganism = function() {
  return this._organism;
};
Project.prototype.setOrganism = function(organism) {
  if (organism !== undefined && organism !== null) {
    this._organism = new Annotation(organism);
  } else {
    this._organism = undefined;
  }
};

Project.prototype.getStatus = function() {
  return this._status;
};
Project.prototype.setStatus = function(status) {
  this._status = status;
};

Project.prototype.getProgress = function() {
  return this._progress;
};
Project.prototype.setProgress = function(progress) {
  this._progress = progress;
};

module.exports = Project;
